<?php declare (strict_types = 1);

namespace USKit\Base\Parts;

use OcArticle;
use Infomaker\Everyware\Support\Date;
use Infomaker\Everyware\Base\Models\Post;
use Infomaker\Everyware\Base\PropertyParser;
use Infomaker\Everyware\Support\NewRelicLog;
use Infomaker\Everyware\Base\ArticleBodyParser;
use Infomaker\Everyware\Base\OcPropertyMapConfig;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\NewsML\NewsMLTransformerManager;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\OpenContent\OpenContentProvider;

/**
 * Class NewsMLArticle
 * @property string     headline
 * @property array      channels
 * @property Date       pubdate
 *
 * @property Collection articles
 * @property Collection authors
 * @property Collection categories
 * @property Collection places
 * @property Collection stories
 * @property Collection tags
 *
 * @package EuKit\Base
 */
class NewsMLArticle extends GenericPropertyObject
{
    public function __construct(PropertyObject $object)
    {
        $this->fill($object->getProperties());
    }

    /**
     * Contains an instance of the parser for the article
     *
     * @var ArticleBodyParser
     */
    protected $parser;

    public function getParsedBody(): ArticleBodyParser
    {
        return $this->getBodyParser();
    }

    public function getBodyParser(): ArticleBodyParser
    {
        if ($this->parser === null) {
            $this->parser = new ArticleBodyParser($this);
        }

        return $this->parser;
    }

    public function getNewsMLTeaser()
    {
        $newsml = $this->teaser_raw;

        if (empty($newsml)) {
            NewRelicLog::error(sprintf(
                'Will not parse empty value from Object: %s',
                $this->get('uuid')
            ));
            return [];
        }

        $transformer = NewsMLTransformerManager::createObjectTransformer();

        $result = $transformer->transform($newsml);

        return count($result) > 0 ? array_shift($result) : null;
    }

    public function hasRelatedArticles(): bool
    {
        return $this->articles->isNotEmpty();
    }

    public function getArticleTags(): Collection
    {
        return $this->getFilteredPlaces()->merge($this->tags)->unique('name')->sortBy('name');
    }

    /**
     * Retrieve a list of Place concepts without the hierarchical relations
     *
     * @since 1.0.0
     * @return Collection
     */
    public function getFilteredPlaces(): Collection
    {
        $places = $this->places;
        $placeRelationUuids = array_column($places->toArray(), 'parent_uuid', 'uuid');

        $placesToKeep = $placeRelationUuids;

        foreach ($placeRelationUuids as $uuid => $parent_uuid) {
            // Remove my parent
            unset($placesToKeep[$parent_uuid]);
        }

        return $places->whereIn('uuid', array_keys($placesToKeep));
    }

    /**
     * Retrieve related article by uuid
     *
     * @param string $uuid
     *
     * @return static|null
     */
    public function getRelatedArticle(string $uuid)
    {
        $articles = $this->articles->whereIn('uuid', $uuid);

        return $articles->isNotEmpty() ? $articles->first() : null;
    }

    public function getPubDate(): Date
    {
        return Date::parse($this->pubdate);
    }

    public static function createFromPost(Post $post, $propertyMap = 'Article')
    {
        $uuid = $post->getMeta('oc_uuid');

        $provider = OpenContentProvider::setup();
        $provider->setPropertyMap($propertyMap);
        $article = $provider->getObject($uuid);

        return new static($article);
    }

    public static function createfromOcObject(OcArticle $article, $propertyMap = 'Article'): self
    {
        $conf = new OcPropertyMapConfig();
        $propertyParser = new PropertyParser($conf);

        return new static($propertyParser->fromOcObject($propertyMap, $article));
    }
}
